@@ -1,15 +1,24 @@
 /* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */
 
 /*
-    Rubber Band
+    Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2010 Chris Cannam.
-    
+    Copyright 2007-2018 Particular Programs Ltd.
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 2 of the
     License, or (at your option) any later version.  See the file
     COPYING included with this distribution for more information.
+
+    Alternatively, if you have a valid commercial licence for the
+    Rubber Band Library obtained by agreement with the copyright
+    holders, you may redistribute and/or modify it under the terms
+    described in that licence.
+
+    If you wish to distribute code using the Rubber Band Library
+    under terms other than those of the GNU General Public License,
+    you must obtain a valid commercial licence before doing so.
 */
 
 #include "StretchCalculator.h"
@@ -36,6 +45,7 @@ StretchCalculator::StretchCalculator(size_t sampleRate,
     m_recovery(0),
     m_prevRatio(1.0),
     m_transientAmnesty(0),
+    m_debugLevel(0),
     m_useHardPeaks(useHardPeaks)
 {
 //    std::cerr << "StretchCalculator::StretchCalculator: useHardPeaks = " << useHardPeaks << std::endl;
@@ -212,7 +222,7 @@ StretchCalculator::mapPeaks(std::vector<Peak> &peaks,
 
     // NB we know for certain we have a mapping from 0 -> 0 (or at
     // least, some mapping for source sample 0) because that is
-    // enforced in setLockPoints above.  However, we aren't guaranteed
+    // enforced in setKeyFrameMap above.  However, we aren't guaranteed
     // to have a mapping for the total duration -- we will usually
     // need to assume it maps to the normal duration * ratio sample
 
@@ -394,6 +404,10 @@ StretchCalculator::reset()
 {
     m_prevDf = 0;
     m_divergence = 0;
+    m_recovery = 0;
+    m_prevRatio = 1.0;
+    m_transientAmnesty = 0;
+    m_keyFrameMap.clear();
 }
 
 std::vector<StretchCalculator::Peak>
