@@ -1,25 +1,35 @@
 /* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */
 
 /*
-    Rubber Band
+    Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2010 Chris Cannam.
-    
+    Copyright 2007-2018 Particular Programs Ltd.
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 2 of the
     License, or (at your option) any later version.  See the file
     COPYING included with this distribution for more information.
+
+    Alternatively, if you have a valid commercial licence for the
+    Rubber Band Library obtained by agreement with the copyright
+    holders, you may redistribute and/or modify it under the terms
+    described in that licence.
+
+    If you wish to distribute code using the Rubber Band Library
+    under terms other than those of the GNU General Public License,
+    you must obtain a valid commercial licence before doing so.
 */
 
 #include "StretcherImpl.h"
 
-#include "dsp/PercussiveAudioCurve.h"
-#include "dsp/HighFrequencyAudioCurve.h"
-#include "dsp/SpectralDifferenceAudioCurve.h"
-#include "dsp/SilentAudioCurve.h"
-#include "dsp/ConstantAudioCurve.h"
-#include "dsp/CompoundAudioCurve.h"
+#include "audiocurves/PercussiveAudioCurve.h"
+#include "audiocurves/HighFrequencyAudioCurve.h"
+#include "audiocurves/SpectralDifferenceAudioCurve.h"
+#include "audiocurves/SilentAudioCurve.h"
+#include "audiocurves/ConstantAudioCurve.h"
+#include "audiocurves/CompoundAudioCurve.h"
+
 #include "dsp/Resampler.h"
 
 #include "StretchCalculator.h"
@@ -35,6 +45,7 @@
 #include <cmath>
 #include <set>
 #include <map>
+#include <algorithm>
 
 using namespace RubberBand;
 
@@ -52,7 +63,7 @@ const size_t
 RubberBandStretcher::Impl::m_defaultIncrement = 256;
 
 const size_t
-RubberBandStretcher::Impl::m_defaultWindowSize = 2048;
+RubberBandStretcher::Impl::m_defaultFftSize = 2048;
 
 int
 RubberBandStretcher::Impl::m_defaultDebugLevel = 0;
@@ -68,19 +79,27 @@ RubberBandStretcher::Impl::Impl(size_t sampleRate,
     m_channels(channels),
     m_timeRatio(initialTimeRatio),
     m_pitchScale(initialPitchScale),
-    m_windowSize(m_defaultWindowSize),
+    m_fftSize(m_defaultFftSize),
+    m_aWindowSize(m_defaultFftSize),
+    m_sWindowSize(m_defaultFftSize),
     m_increment(m_defaultIncrement),
-    m_outbufSize(m_defaultWindowSize * 2),
-    m_maxProcessSize(m_defaultWindowSize),
+    m_outbufSize(m_defaultFftSize * 2),
+    m_maxProcessSize(m_defaultFftSize),
     m_expectedInputDuration(0),
+#ifndef NO_THREADING
     m_threaded(false),
+#endif
     m_realtime(false),
     m_options(options),
     m_debugLevel(m_defaultDebugLevel),
     m_mode(JustCreated),
-    m_window(0),
+    m_awindow(0),
+    m_afilter(0),
+    m_swindow(0),
     m_studyFFT(0),
+#ifndef NO_THREADING
     m_spaceAvailable("space"),
+#endif
     m_inputDuration(0),
     m_detectorType(CompoundAudioCurve::CompoundDetector),
     m_silentHistory(0),
@@ -94,7 +113,7 @@ RubberBandStretcher::Impl::Impl(size_t sampleRate,
     m_freq0(600),
     m_freq1(1200),
     m_freq2(12000),
-    m_baseWindowSize(m_defaultWindowSize)
+    m_baseFftSize(m_defaultFftSize)
 {
     if (!_initialised) {
         system_specific_initialise();
@@ -109,25 +128,27 @@ RubberBandStretcher::Impl::Impl(size_t sampleRate,
     // we don't let it drop below the 48k default
     m_rateMultiple = float(m_sampleRate) / 48000.f;
 //    if (m_rateMultiple < 1.f) m_rateMultiple = 1.f;
-    m_baseWindowSize = roundUp(int(m_defaultWindowSize * m_rateMultiple));
+    m_baseFftSize = roundUp(int(m_defaultFftSize * m_rateMultiple));
 
     if ((options & OptionWindowShort) || (options & OptionWindowLong)) {
         if ((options & OptionWindowShort) && (options & OptionWindowLong)) {
             cerr << "RubberBandStretcher::Impl::Impl: Cannot specify OptionWindowLong and OptionWindowShort together; falling back to OptionWindowStandard" << endl;
         } else if (options & OptionWindowShort) {
-            m_baseWindowSize = m_baseWindowSize / 2;
+            m_baseFftSize = m_baseFftSize / 2;
             if (m_debugLevel > 0) {
-                cerr << "setting baseWindowSize to " << m_baseWindowSize << endl;
+                cerr << "setting baseFftSize to " << m_baseFftSize << endl;
             }
         } else if (options & OptionWindowLong) {
-            m_baseWindowSize = m_baseWindowSize * 2;
+            m_baseFftSize = m_baseFftSize * 2;
             if (m_debugLevel > 0) {
-                cerr << "setting baseWindowSize to " << m_baseWindowSize << endl;
+                cerr << "setting baseFftSize to " << m_baseFftSize << endl;
             }
         }
-        m_windowSize = m_baseWindowSize;
-        m_outbufSize = m_baseWindowSize * 2;
-        m_maxProcessSize = m_baseWindowSize;
+        m_fftSize = m_baseFftSize;
+        m_aWindowSize = m_baseFftSize;
+        m_sWindowSize = m_baseFftSize;
+        m_outbufSize = m_sWindowSize * 2;
+        m_maxProcessSize = m_aWindowSize;
     }
 
     if (m_options & OptionProcessRealTime) {
@@ -139,6 +160,7 @@ RubberBandStretcher::Impl::Impl(size_t sampleRate,
         }
     }
 
+#ifndef NO_THREADING
     if (m_channels > 1) {
 
         m_threaded = true;
@@ -156,12 +178,14 @@ RubberBandStretcher::Impl::Impl(size_t sampleRate,
             cerr << "Going multithreaded..." << endl;
         }
     }
+#endif
 
     configure();
 }
 
 RubberBandStretcher::Impl::~Impl()
 {
+#ifndef NO_THREADING
     if (m_threaded) {
         MutexLocker locker(&m_threadSetMutex);
         for (set<ProcessThread *>::iterator i = m_threadSet.begin();
@@ -174,6 +198,7 @@ RubberBandStretcher::Impl::~Impl()
             delete *i;
         }
     }
+#endif
 
     for (size_t c = 0; c < m_channels; ++c) {
         delete m_channelData[c];
@@ -189,11 +214,16 @@ RubberBandStretcher::Impl::~Impl()
          i != m_windows.end(); ++i) {
         delete i->second;
     }
+    for (map<size_t, SincWindow<float> *>::iterator i = m_sincs.begin();
+         i != m_sincs.end(); ++i) {
+        delete i->second;
+    }
 }
 
 void
 RubberBandStretcher::Impl::reset()
 {
+#ifndef NO_THREADING
     if (m_threaded) {
         m_threadSetMutex.lock();
         for (set<ProcessThread *>::iterator i = m_threadSet.begin();
@@ -207,11 +237,12 @@ RubberBandStretcher::Impl::reset()
         }
         m_threadSet.clear();
     }
+#endif
 
     m_emergencyScavenger.scavenge();
 
     if (m_stretchCalculator) {
-        m_stretchCalculator->setKeyFrameMap(std::map<size_t, size_t>());
+        m_stretchCalculator->reset();
     }
 
     for (size_t c = 0; c < m_channels; ++c) {
@@ -225,7 +256,9 @@ RubberBandStretcher::Impl::reset()
     m_inputDuration = 0;
     m_silentHistory = 0;
 
+#ifndef NO_THREADING
     if (m_threaded) m_threadSetMutex.unlock();
+#endif
 
     reconfigure();
 }
@@ -378,7 +411,7 @@ void
 RubberBandStretcher::Impl::calculateSizes()
 {
     size_t inputIncrement = m_defaultIncrement;
-    size_t windowSize = m_baseWindowSize;
+    size_t windowSize = m_baseFftSize;
     size_t outputIncrement;
 
     if (m_pitchScale <= 0.0) {
@@ -413,7 +446,7 @@ RubberBandStretcher::Impl::calculateSizes()
             if (outputIncrement < m_defaultIncrement / 4) {
                 if (outputIncrement < 1) outputIncrement = 1;
                 while (outputIncrement < m_defaultIncrement / 4 &&
-                       windowSize < m_baseWindowSize * 4) {
+                       windowSize < m_baseFftSize * 4) {
                     outputIncrement *= 2;
                     inputIncrement = lrint(ceil(outputIncrement / r));
                     windowSize = roundUp(lrint(ceil(inputIncrement * windowIncrRatio)));
@@ -482,10 +515,19 @@ RubberBandStretcher::Impl::calculateSizes()
         }
     }
 
-    // windowSize can be almost anything, but it can't be greater than
-    // 4 * m_baseWindowSize unless ratio is less than 1/1024.
+    // m_fftSize can be almost anything, but it can't be greater than
+    // 4 * m_baseFftSize unless ratio is less than 1/1024.
+
+    m_fftSize = windowSize;
+    
+    if (m_options & OptionSmoothingOn) {
+        m_aWindowSize = windowSize * 2;
+        m_sWindowSize = windowSize * 2;
+    } else {
+        m_aWindowSize = windowSize;
+        m_sWindowSize = windowSize;
+    }
 
-    m_windowSize = windowSize;
     m_increment = inputIncrement;
 
     // When squashing, the greatest theoretically possible output
@@ -496,25 +538,26 @@ RubberBandStretcher::Impl::calculateSizes()
     // ratio) for any chunk.
 
     if (m_debugLevel > 0) {
-        cerr << "configure: effective ratio = " << getEffectiveRatio() << endl;
-        cerr << "configure: window size = " << m_windowSize << ", increment = " << m_increment << " (approx output increment = " << int(lrint(m_increment * getEffectiveRatio())) << ")" << endl;
+        cerr << "configure: time ratio = " << m_timeRatio << ", pitch scale = " << m_pitchScale << ", effective ratio = " << getEffectiveRatio() << endl;
+        cerr << "configure: analysis window size = " << m_aWindowSize << ", synthesis window size = " << m_sWindowSize << ", fft size = " << m_fftSize << ", increment = " << m_increment << " (approx output increment = " << int(lrint(m_increment * getEffectiveRatio())) << ")" << endl;
     }
 
-    if (m_windowSize > m_maxProcessSize) {
-        m_maxProcessSize = m_windowSize;
+    if (std::max(m_aWindowSize, m_sWindowSize) > m_maxProcessSize) {
+        m_maxProcessSize = std::max(m_aWindowSize, m_sWindowSize);
     }
 
     m_outbufSize =
         size_t
         (ceil(max
               (m_maxProcessSize / m_pitchScale,
-               m_windowSize * 2 * (m_timeRatio > 1.f ? m_timeRatio : 1.f))));
+               m_maxProcessSize * 2 * (m_timeRatio > 1.f ? m_timeRatio : 1.f))));
 
     if (m_realtime) {
         // This headroom is so as to try to avoid reallocation when
         // the pitch scale changes
         m_outbufSize = m_outbufSize * 16;
     } else {
+#ifndef NO_THREADING
         if (m_threaded) {
             // This headroom is to permit the processing threads to
             // run ahead of the buffer output drainage; the exact
@@ -522,6 +565,7 @@ RubberBandStretcher::Impl::calculateSizes()
             // results
             m_outbufSize = m_outbufSize * 16;
         }
+#endif
     }
 
     if (m_debugLevel > 0) {
@@ -535,16 +579,22 @@ RubberBandStretcher::Impl::configure()
 //    std::cerr << "configure[" << this << "]: realtime = " << m_realtime << ", pitch scale = "
 //              << m_pitchScale << ", channels = " << m_channels << std::endl;
 
-    size_t prevWindowSize = m_windowSize;
+    size_t prevFftSize = m_fftSize;
+    size_t prevAWindowSize = m_aWindowSize;
+    size_t prevSWindowSize = m_sWindowSize;
     size_t prevOutbufSize = m_outbufSize;
     if (m_windows.empty()) {
-        prevWindowSize = 0;
+        prevFftSize = 0;
+        prevAWindowSize = 0;
+        prevSWindowSize = 0;
         prevOutbufSize = 0;
     }
 
     calculateSizes();
 
-    bool windowSizeChanged = (prevWindowSize != m_windowSize);
+    bool fftSizeChanged = (prevFftSize != m_fftSize);
+    bool windowSizeChanged = ((prevAWindowSize != m_aWindowSize) ||
+                              (prevSWindowSize != m_sWindowSize));
     bool outbufSizeChanged = (prevOutbufSize != m_outbufSize);
 
     // This function may be called at any time in non-RT mode, after a
@@ -557,12 +607,14 @@ RubberBandStretcher::Impl::configure()
 
     set<size_t> windowSizes;
     if (m_realtime) {
-        windowSizes.insert(m_baseWindowSize);
-        windowSizes.insert(m_baseWindowSize / 2);
-        windowSizes.insert(m_baseWindowSize * 2);
-//        windowSizes.insert(m_baseWindowSize * 4);
+        windowSizes.insert(m_baseFftSize);
+        windowSizes.insert(m_baseFftSize / 2);
+        windowSizes.insert(m_baseFftSize * 2);
+//        windowSizes.insert(m_baseFftSize * 4);
     }
-    windowSizes.insert(m_windowSize);
+    windowSizes.insert(m_fftSize);
+    windowSizes.insert(m_aWindowSize);
+    windowSizes.insert(m_sWindowSize);
 
     if (windowSizeChanged) {
 
@@ -571,11 +623,16 @@ RubberBandStretcher::Impl::configure()
             if (m_windows.find(*i) == m_windows.end()) {
                 m_windows[*i] = new Window<float>(HanningWindow, *i);
             }
+            if (m_sincs.find(*i) == m_sincs.end()) {
+                m_sincs[*i] = new SincWindow<float>(*i, *i);
+            }
         }
-        m_window = m_windows[m_windowSize];
+        m_awindow = m_windows[m_aWindowSize];
+        m_afilter = m_sincs[m_aWindowSize];
+        m_swindow = m_windows[m_sWindowSize];
 
         if (m_debugLevel > 0) {
-            cerr << "Window area: " << m_window->getArea() << "; synthesis window area: " << m_window->getArea() << endl;
+            cerr << "Window area: " << m_awindow->getArea() << "; synthesis window area: " << m_swindow->getArea() << endl;
         }
     }
 
@@ -588,13 +645,16 @@ RubberBandStretcher::Impl::configure()
 
         for (size_t c = 0; c < m_channels; ++c) {
             m_channelData.push_back
-                (new ChannelData(windowSizes, 1, m_windowSize, m_outbufSize));
+                (new ChannelData(windowSizes,
+                                 std::max(m_aWindowSize, m_sWindowSize),
+                                 m_fftSize,
+                                 m_outbufSize));
         }
     }
 
-    if (!m_realtime && windowSizeChanged) {
+    if (!m_realtime && fftSizeChanged) {
         delete m_studyFFT;
-        m_studyFFT = new FFT(m_windowSize, m_debugLevel);
+        m_studyFFT = new FFT(m_fftSize, m_debugLevel);
         m_studyFFT->initFloat();
     }
 
@@ -626,21 +686,21 @@ RubberBandStretcher::Impl::configure()
 
     delete m_phaseResetAudioCurve;
     m_phaseResetAudioCurve = new CompoundAudioCurve
-        (CompoundAudioCurve::Parameters(m_sampleRate, m_windowSize));
+        (CompoundAudioCurve::Parameters(m_sampleRate, m_fftSize));
     m_phaseResetAudioCurve->setType(m_detectorType);
 
     delete m_silentAudioCurve;
     m_silentAudioCurve = new SilentAudioCurve
-        (SilentAudioCurve::Parameters(m_sampleRate, m_windowSize));
+        (SilentAudioCurve::Parameters(m_sampleRate, m_fftSize));
 
     if (!m_realtime) {
         delete m_stretchAudioCurve;
         if (!(m_options & OptionStretchPrecise)) {
             m_stretchAudioCurve = new SpectralDifferenceAudioCurve
-                (SpectralDifferenceAudioCurve::Parameters(m_sampleRate, m_windowSize));
+                (SpectralDifferenceAudioCurve::Parameters(m_sampleRate, m_fftSize));
         } else {
             m_stretchAudioCurve = new ConstantAudioCurve
-                (ConstantAudioCurve::Parameters(m_sampleRate, m_windowSize));
+                (ConstantAudioCurve::Parameters(m_sampleRate, m_fftSize));
         }
     }
 
@@ -664,9 +724,12 @@ RubberBandStretcher::Impl::configure()
     // want gaps when the ratio changes.
 
     if (!m_realtime) {
+        if (m_debugLevel > 1) {
+            cerr << "Not real time mode: prefilling" << endl;
+        }
         for (size_t c = 0; c < m_channels; ++c) {
             m_channelData[c]->reset();
-            m_channelData[c]->inbuf->zero(m_windowSize/2);
+            m_channelData[c]->inbuf->zero(m_aWindowSize/2);
         }
     }
 }
@@ -688,7 +751,9 @@ RubberBandStretcher::Impl::reconfigure()
         configure();
     }
 
-    size_t prevWindowSize = m_windowSize;
+    size_t prevFftSize = m_fftSize;
+    size_t prevAWindowSize = m_aWindowSize;
+    size_t prevSWindowSize = m_sWindowSize;
     size_t prevOutbufSize = m_outbufSize;
 
     calculateSizes();
@@ -698,18 +763,34 @@ RubberBandStretcher::Impl::reconfigure()
     // where not all of the things we need were correctly created when
     // we first configured (for whatever reason).  This is intended to
     // be "effectively" realtime safe.  The same goes for
-    // ChannelData::setOutbufSize and setWindowSize.
+    // ChannelData::setOutbufSize and setSizes.
 
-    if (m_windowSize != prevWindowSize) {
+    if (m_aWindowSize != prevAWindowSize ||
+        m_sWindowSize != prevSWindowSize) {
 
-        if (m_windows.find(m_windowSize) == m_windows.end()) {
-            std::cerr << "WARNING: reconfigure(): window allocation (size " << m_windowSize << ") required in RT mode" << std::endl;
-            m_windows[m_windowSize] = new Window<float>(HanningWindow, m_windowSize);
+        if (m_windows.find(m_aWindowSize) == m_windows.end()) {
+            std::cerr << "WARNING: reconfigure(): window allocation (size " << m_aWindowSize << ") required in RT mode" << std::endl;
+            m_windows[m_aWindowSize] = new Window<float>
+                (HanningWindow, m_aWindowSize);
+            m_sincs[m_aWindowSize] = new SincWindow<float>
+                (m_aWindowSize, m_aWindowSize);
         }
-        m_window = m_windows[m_windowSize];
+
+        if (m_windows.find(m_sWindowSize) == m_windows.end()) {
+            std::cerr << "WARNING: reconfigure(): window allocation (size " << m_sWindowSize << ") required in RT mode" << std::endl;
+            m_windows[m_sWindowSize] = new Window<float>
+                (HanningWindow, m_sWindowSize);
+            m_sincs[m_sWindowSize] = new SincWindow<float>
+                (m_sWindowSize, m_sWindowSize);
+        }
+
+        m_awindow = m_windows[m_aWindowSize];
+        m_afilter = m_sincs[m_aWindowSize];
+        m_swindow = m_windows[m_sWindowSize];
 
         for (size_t c = 0; c < m_channels; ++c) {
-            m_channelData[c]->setWindowSize(m_windowSize);
+            m_channelData[c]->setSizes(std::max(m_aWindowSize, m_sWindowSize),
+                                       m_fftSize);
         }
     }
 
@@ -727,16 +808,18 @@ RubberBandStretcher::Impl::reconfigure()
             std::cerr << "WARNING: reconfigure(): resampler construction required in RT mode" << std::endl;
 
             m_channelData[c]->resampler =
-                new Resampler(Resampler::FastestTolerable, 1, m_windowSize,
+                new Resampler(Resampler::FastestTolerable, 1, m_sWindowSize,
                               m_debugLevel);
 
-            m_channelData[c]->setResampleBufSize
-                (lrintf(ceil((m_increment * m_timeRatio * 2) / m_pitchScale)));
+            size_t rbs = 
+                lrintf(ceil((m_increment * m_timeRatio * 2) / m_pitchScale));
+            if (rbs < m_increment * 16) rbs = m_increment * 16;
+            m_channelData[c]->setResampleBufSize(rbs);
         }
     }
 
-    if (m_windowSize != prevWindowSize) {
-        m_phaseResetAudioCurve->setWindowSize(m_windowSize);
+    if (m_fftSize != prevFftSize) {
+        m_phaseResetAudioCurve->setFftSize(m_fftSize);
     }
 }
 
@@ -744,7 +827,7 @@ size_t
 RubberBandStretcher::Impl::getLatency() const
 {
     if (!m_realtime) return 0;
-    return int((m_windowSize/2) / m_pitchScale + 1);
+    return int((m_aWindowSize/2) / m_pitchScale + 1);
 }
 
 void
@@ -887,24 +970,51 @@ RubberBandStretcher::Impl::study(const float *const *input, size_t samples, bool
             consumed += writable;
         }
 
-	while ((inbuf.getReadSpace() >= int(m_windowSize)) ||
-               (final && (inbuf.getReadSpace() >= int(m_windowSize/2)))) {
+	while ((inbuf.getReadSpace() >= int(m_aWindowSize)) ||
+               (final && (inbuf.getReadSpace() >= int(m_aWindowSize/2)))) {
 
-	    // We know we have at least m_windowSize samples available
-	    // in m_inbuf.  We need to peek m_windowSize of them for
-	    // processing, and then skip m_increment to advance the
-	    // read pointer.
+	    // We know we have at least m_aWindowSize samples
+	    // available in m_inbuf.  We need to peek m_aWindowSize of
+	    // them for processing, and then skip m_increment to
+	    // advance the read pointer.
 
             // cd.accumulator is not otherwise used during studying,
             // so we can use it as a temporary buffer here
 
-            size_t got = inbuf.peek(cd.accumulator, m_windowSize);
-            assert(final || got == m_windowSize);
+            size_t ready = inbuf.getReadSpace();
+            assert(final || ready >= m_aWindowSize);
+            inbuf.peek(cd.accumulator, std::min(ready, m_aWindowSize));
 
-            m_window->cut(cd.accumulator);
+            if (m_aWindowSize == m_fftSize) {
 
-            // We don't need the fftshift for studying, as we're only
-            // interested in magnitude
+                // We don't need the fftshift for studying, as we're
+                // only interested in magnitude.
+
+                m_awindow->cut(cd.accumulator);
+
+            } else {
+
+                // If we need to fold (i.e. if the window size is
+                // greater than the fft size so we are doing a
+                // time-aliased presum fft) or zero-pad, then we might
+                // as well use our standard function for it.  This
+                // means we retain the m_afilter cut if folding as well,
+                // which is good for consistency with real-time mode.
+                // We get fftshift as well, which we don't want, but
+                // the penalty is nominal.
+
+                // Note that we can't do this in-place.  Pity
+
+                float *tmp = (float *)alloca
+                    (std::max(m_fftSize, m_aWindowSize) * sizeof(float));
+
+                if (m_aWindowSize > m_fftSize) {
+                    m_afilter->cut(cd.accumulator);
+                }
+
+                cutShiftAndFold(tmp, m_fftSize, cd.accumulator, m_awindow);
+                v_copy(cd.accumulator, tmp, m_fftSize);
+            }
 
             m_studyFFT->forwardMagnitude(cd.accumulator, cd.fltbuf);
 
@@ -925,12 +1035,12 @@ RubberBandStretcher::Impl::study(const float *const *input, size_t samples, bool
 
 //            cout << df << endl;
 
-            // We have augmented the input by m_windowSize/2 so
-            // that the first chunk is centred on the first audio
-            // sample.  We want to ensure that m_inputDuration
-            // contains the exact input duration without including
-            // this extra bit.  We just add up all the increments
-            // here, and deduct the extra afterwards.
+            // We have augmented the input by m_aWindowSize/2 so that
+            // the first chunk is centred on the first audio sample.
+            // We want to ensure that m_inputDuration contains the
+            // exact input duration without including this extra bit.
+            // We just add up all the increments here, and deduct the
+            // extra afterwards.
 
             m_inputDuration += m_increment;
 //                cerr << "incr input duration by increment: " << m_increment << " -> " << m_inputDuration << endl;
@@ -943,12 +1053,12 @@ RubberBandStretcher::Impl::study(const float *const *input, size_t samples, bool
         m_inputDuration += rs;
 //        cerr << "incr input duration by read space: " << rs << " -> " << m_inputDuration << endl;
 
-        if (m_inputDuration > m_windowSize/2) { // deducting the extra
-            m_inputDuration -= m_windowSize/2;
+        if (m_inputDuration > m_aWindowSize/2) { // deducting the extra
+            m_inputDuration -= m_aWindowSize/2;
         }
     }
 
-    if (m_channels > 1) delete[] mdalloc;
+    if (m_channels > 1 || final) delete[] mdalloc;
 }
 
 vector<int>
@@ -1009,11 +1119,15 @@ RubberBandStretcher::Impl::calculateStretch()
 
     double prdm = 0, sdm = 0;
     if (!m_phaseResetDf.empty()) {
-        for (int i = 0; i < m_phaseResetDf.size(); ++i) prdm += m_phaseResetDf[i];
+        for (int i = 0; i < (int)m_phaseResetDf.size(); ++i) {
+            prdm += m_phaseResetDf[i];
+        }
         prdm /= m_phaseResetDf.size();
     }
     if (!m_stretchDf.empty()) {
-        for (int i = 0; i < m_stretchDf.size(); ++i) sdm += m_stretchDf[i];
+        for (int i = 0; i < (int)m_stretchDf.size(); ++i) {
+            sdm += m_stretchDf[i];
+        }
         sdm /= m_stretchDf.size();
     }
 //    std::cerr << "phase reset df mean = " << prdm << ", stretch df mean = " << sdm << std::endl;
@@ -1029,7 +1143,7 @@ RubberBandStretcher::Impl::calculateStretch()
         if (i >= m_silence.size()) break;
         if (m_silence[i]) ++history;
         else history = 0;
-        if (history >= int(m_windowSize / m_increment) && increments[i] >= 0) {
+        if (history >= int(m_aWindowSize / m_increment) && increments[i] >= 0) {
             increments[i] = -increments[i];
             if (m_debugLevel > 1) {
                 std::cerr << "phase reset on silence (silent history == "
@@ -1068,21 +1182,35 @@ RubberBandStretcher::Impl::getSamplesRequired() const
 
         ChannelData &cd = *m_channelData[c];
         RingBuffer<float> &inbuf = *cd.inbuf;
+        RingBuffer<float> &outbuf = *cd.outbuf;
 
         size_t rs = inbuf.getReadSpace();
+        size_t ws = outbuf.getReadSpace();
+
+        if (m_debugLevel > 2) {
+            cerr << "getSamplesRequired: ws = " << ws << ", rs = " << rs << ", m_aWindowSize = " << m_aWindowSize << endl;
+        }
+
+        // We should never return zero in non-threaded modes if
+        // available() would also return zero, i.e. if ws == 0.  If we
+        // do that, nothing will ever happen again!  We need to demand
+        // at least one increment (i.e. a nominal amount) to feed the
+        // engine.
+
+        if (ws == 0 && reqd == 0) reqd = m_increment;
 
         // See notes in testInbufReadSpace 
 
-        if (rs < m_windowSize && !cd.draining) {
+        if (rs < m_aWindowSize && !cd.draining) {
             
             if (cd.inputSize == -1) {
-                reqdHere = m_windowSize - rs;
+                reqdHere = m_aWindowSize - rs;
                 if (reqdHere > reqd) reqd = reqdHere;
                 continue;
             }
         
             if (rs == 0) {
-                reqdHere = m_windowSize;
+                reqdHere = m_aWindowSize;
                 if (reqdHere > reqd) reqd = reqdHere;
                 continue;
             }
@@ -1105,14 +1233,23 @@ RubberBandStretcher::Impl::process(const float *const *input, size_t samples, bo
     if (m_mode == JustCreated || m_mode == Studying) {
 
         if (m_mode == Studying) {
+
             calculateStretch();
+
+            if (!m_realtime) {
+                // See note in configure() above. Of course, we should
+                // never enter Studying unless we are non-RT anyway
+                if (m_debugLevel > 1) {
+                    cerr << "Not real time mode: prefilling" << endl;
+                }
+                for (size_t c = 0; c < m_channels; ++c) {
+                    m_channelData[c]->reset();
+                    m_channelData[c]->inbuf->zero(m_aWindowSize/2);
+                }
+            }
         }
 
-        for (size_t c = 0; c < m_channels; ++c) {
-            m_channelData[c]->reset();
-            m_channelData[c]->inbuf->zero(m_windowSize/2);
-        }
-
+#ifndef NO_THREADING
         if (m_threaded) {
             MutexLocker locker(&m_threadSetMutex);
 
@@ -1126,6 +1263,7 @@ RubberBandStretcher::Impl::process(const float *const *input, size_t samples, bo
                 cerr << m_channels << " threads created" << endl;
             }
         }
+#endif
         
         m_mode = Processing;
     }
@@ -1139,12 +1277,6 @@ RubberBandStretcher::Impl::process(const float *const *input, size_t samples, bo
 
     while (!allConsumed) {
 
-//#ifndef NO_THREADING
-//        if (m_threaded) {
-//            pthread_mutex_lock(&m_inputProcessedMutex);
-//        }
-//#endif
-
         // In a threaded mode, our "consumed" counters only indicate
         // the number of samples that have been taken into the input
         // ring buffers waiting to be processed by the process thread.
@@ -1155,7 +1287,8 @@ RubberBandStretcher::Impl::process(const float *const *input, size_t samples, bo
 
         for (size_t c = 0; c < m_channels; ++c) {
             consumed[c] += consumeChannel(c,
-                                          input[c] + consumed[c],
+                                          input,
+                                          consumed[c],
                                           samples - consumed[c],
                                           final);
             if (consumed[c] < samples) {
@@ -1167,7 +1300,11 @@ RubberBandStretcher::Impl::process(const float *const *input, size_t samples, bo
                 }
 //                cerr << "process: happy with channel " << c << endl;
             }
-            if (!m_threaded && !m_realtime) {
+            if (
+#ifndef NO_THREADING
+                !m_threaded &&
+#endif
+                !m_realtime) {
                 bool any = false, last = false;
                 processChunks(c, any, last);
             }
@@ -1180,7 +1317,7 @@ RubberBandStretcher::Impl::process(const float *const *input, size_t samples, bo
             // the realtime onset detector
             processOneChunk();
         }
-
+#ifndef NO_THREADING
         if (m_threaded) {
             for (ThreadSet::iterator i = m_threadSet.begin();
                  i != m_threadSet.end(); ++i) {
@@ -1191,23 +1328,17 @@ RubberBandStretcher::Impl::process(const float *const *input, size_t samples, bo
                 m_spaceAvailable.wait(500);
             }
             m_spaceAvailable.unlock();
-/*
-        } else {
-            if (!allConsumed) {
-                cerr << "RubberBandStretcher::Impl::process: ERROR: Too much data provided to process() call -- either call setMaxProcessSize() beforehand, or provide only getSamplesRequired() frames at a time" << endl;
-                for (size_t c = 0; c < m_channels; ++c) {
-                    cerr << "channel " << c << ": " << samples << " provided, " << consumed[c] << " consumed" << endl;
-                }
-//                break;
-            }
-*/
         }
+#endif
 
-//        if (!allConsumed) cerr << "process looping" << endl;
+        if (m_debugLevel > 2) {
+            if (!allConsumed) cerr << "process looping" << endl;
+        }
+    }
 
+    if (m_debugLevel > 2) {
+        cerr << "process returning" << endl;
     }
-    
-//    cerr << "process returning" << endl;
 
     if (final) m_mode = Finished;
 }
